<?php

/**
 * Copyright 2014 Fabian Grutschus. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those
 * of the authors and should not be interpreted as representing official policies,
 * either expressed or implied, of the copyright holders.
 *
 * @author    Fabian Grutschus <f.grutschus@lubyte.de>
 * @copyright 2014 Fabian Grutschus. All rights reserved.
 * @license   BSD
 * @link      http://github.com/fabiang/xmpp
 */

namespace Fabiang\Xmpp\Util;

use PHPUnit\Framework\TestCase;

/**
 * Generated by PHPUnit_SkeletonGenerator on 2014-10-29 at 11:04:03.
 *
 * @coversDefaultClass Fabiang\Xmpp\Util\XML
 */
class XMLTest extends TestCase
{

    /**
     * @covers ::quoteMessage
     * @uses Fabiang\Xmpp\Util\XML::quote
     */
    public function testQuoteMessage()
    {
        $this->assertSame(
            '<foo lt="&lt;" gt="&gt;", amp="&amp;" quot="&quot;">',
            XML::quoteMessage('<foo lt="%s" gt="%s", amp="%s" quot="%s">', '<', '>', '&', '"')
        );
    }

    /**
     * @covers ::quoteMessage
     * @uses Fabiang\Xmpp\Util\XML::quote
     */
    public function testQuoteMessageNoVariables()
    {
        $this->assertSame('<foo>', XML::quoteMessage('<foo>'));
    }

    /**
     * @covers ::quote
     * @uses Fabiang\Xmpp\Util\XML::quote
     */
    public function testQuote()
    {
        $this->assertSame('&lt;foo&gt;', XML::quote('<foo>'));
    }

    /**
     * @covers ::quote
     * @uses Fabiang\Xmpp\Util\XML::quote
     * @requires PHP 5.4
     */
    public function testQuoteAsXML()
    {
        $this->assertSame('&apos;', XML::quote("'"));
    }

    /**
     * @covers ::generateId
     * @uses Fabiang\Xmpp\Util\XML::quote
     */
    public function testGenerateId()
    {
        $this->assertStringStartsWith('fabiang_xmpp_', XML::generateId());
    }

    /**
     * @covers ::base64Encode
     * @covers ::base64Decode
     * @uses Fabiang\Xmpp\Util\XML::quote
     */
    public function testBase64()
    {
        $this->assertSame('test', XML::base64Decode(XML::base64Encode('test')));
    }
}
